# -*- coding: UTF-8 -*-

#!! ベクトル演算関係の関数集<br>2024.3.11 def check_parallel2D(v0, v1)を追加

# 使用例
# import my_vector as myv

# 履歴
# 2021.8.8  vvth_pm(v0,v1)  2つのVectorの内角を正負付で返す。v0 -> v1の回転角を正
# 2022.2.21 v2p3(v3)    # xyz直交座標->rθφ極座標変換 戻り値:np.array() rad & deg
#　                 y_を法線とする平面上にx_をθ回転後φ回転させる（反時計回）
# 2022.5.24 get_sva(sl, sl100, dt, flgp = 0):  # flgp = 1:mgraph 表示  追加
#           sl, sl100より速度、加速度を求める関数(0-1補間付)  flgp=1: グラフ表示（省略可) 
# 2022.6.14 平面と点の距離を測定  f2plen(v0_, p0, p1):      return length
#           点p0を含み、法線Vector v0_ を持つ平面と点p1との距離
# 2024.3.11 def check_parallel2D(v0, v1):   # v0とv1が平行かどうかをチェック
# 2024.7.3  def ej(th):  # e^jθを返す関数　戻り値は複素数(complex)
#           def c2v(c):  # 複素数をnp.array Vectorに変換
# 2024.7.24 def vvth_pm:   第2現象で挙動不審を修正

import sys
import math
import cmath
import numpy as np
from numpy import linalg as LA

#平面と点の距離を測定
#  点p0を含み、法線Vector v0 を持つ平面と点p1との距離
def f2plen(v0, p0, p1):
    a, b, c = v0[0], v0[1], v0[2]
    d = -(a*p0[0] + b*p0[1] + c*p0[2])
    tmp = a*p1[0] + b*p1[1] + c*p1[2]
    
    length = abs(tmp+d) / LA.norm(v0)
    return length

# p0を通る (p1-p2直線の法線Vector v0x_) と その交点 px, 流れ方向単位Vector vx2_を返す
def l2pvec(p0, p1, p2): 
    v10= p0 - p1; v12= p2 - p1
    vx2_= v12/(LA.norm(v12))
    th= vvth(v12,v10)
    #th= vvth_pm(v12,v10)
    tmp= LA.norm(v10)*np.cos(th)
    v1x= tmp*vx2_
    px= p1 + v1x
    tmp= LA.norm(v10)*np.sin(th)
    v0x_= (px - p0)/tmp

    return (px, v0x_, vx2_)


#iCAD的直交座標を極座標に変換 
def v2p3(v3):  # xyz直交座標->rθφ極座標変換 x_をxz平面で反時計回りにθ回転後y軸方向にφ回転 
    x = v3[0]; y = v3[1]; z = v3[2]
    r0 = np.sqrt(x**2+y**2+z**2)
    r1 = np.sqrt(x**2+z**2)
    th = math.atan2(-z, x)
    fai = np.arccos(np.clip(r1/r0, -1.0, 1.0))
    p_rad = np.array([r0, th, fai])
    p_deg = np.array([r0, np.rad2deg(th), np.rad2deg(fai)])
    
    return (p_rad, p_deg)


# ベクトルv0をth[rad]回転したベクトルv1を返す(2D)
def rotate2(v0,th):     
    a = np.array([[np.cos(th), -np.sin(th)],
                  [np.sin(th),  np.cos(th)]])
    v1 = np.dot(a, v0)        
    return v1

# Vector v0 と v1 の内角を求める(2D,3D) -π～π[rad]
def vvth(v0,v1):   # [rad]  戻り値は正
    tmp = np.dot(v0,v1) / LA.norm(v0) / LA.norm(v1)
    th = np.arccos(np.clip(tmp, -1.0, 1.0))
    #print(tmp,th)
    return th




# 2023.7.27  内積がほぼ 0 の時 0 を返す gosa を追加
#   したら、icd_bolt_set.pyで不具合発生!!!
"""
def vvth(v0, v1, gosa= 0.000001):   # [rad]  戻り値は正
    v0_= v0/LA.norm(v0); v1_= v1/LA.norm(v1)
    if abs(v0_[0]- v1_[0]) < gosa:
        th= 0.0
    else:
        tmp = np.dot(v0,v1) / LA.norm(v0) / LA.norm(v1)
        th = np.arccos(np.clip(tmp, -1.0, 1.0))
        #print(tmp,th)
    return th

def vvth_pm(v0,v1,gosa= 0.000001):     # 2つのVectorの内角を正負付で返す。v0 -> v1の回転角を正
    th = vvth(v0, v1, gosa)
    v2 = rotate2(v0, th)    # v0をθ回転して
    lp = LA.norm(v1-v2)
    lm = LA.norm(v1-v0)     # lp <= lm ならθは正

    if lp <= lm:
        return th
    else:
        return -th
"""
    

def vvth_pm_bak(v0,v1):     # 2つのVectorの内角を正負付で返す。v0 -> v1の回転角を正
    th = vvth(v0, v1)
    v2 = rotate2(v0, th)    # v0をθ回転して
    lp = LA.norm(v1-v2)
    lm = LA.norm(v1-v0)     # lp <= lm ならθは正

    if lp <= lm:
        return th
    else:
        return -th

# 2024.7.24  第2現象で挙動不審を修正
def vvth_pm(v0,v1):     # 2つのVectorの内角を正負付で返す。v0 -> v1の回転角を正
    th = vvth(v0, v1)
    v2 = rotate2(v0, th)    # v0をθ回転して
    v3 = rotate2(v0, -th)    # v0を-θ回転して
    lp = LA.norm(v1-v2)
    lm = LA.norm(v1-v3)     # lp <= lm ならθは正
    
    if lp <= lm:
        return th
    else:
        return -th

#--------------------------------
#/*  回転テンソル  */#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#--------------------------------

#/*  Ekth    */
#/* k(z)軸回りに a_3d を th_3d [deg] 回転(反時計回) => c_3d  */
def ekth(a_3d,th_3d):
    e_3d[0][0] = math.cos(th_3d); e_3d[0][1] = -math.sin(th_3d); e_3d[0][2] = 0.0  
    e_3d[1][0] = math.sin(th_3d); e_3d[1][1] =  math.cos(th_3d); e_3d[1][2] = 0.0
    e_3d[2][0] = 0.0         ; e_3d[2][1] =  0.0         ; e_3d[2][2] = 1
    mtrx3(a_3d,e_3d)
    #name_3d = "mtrx3";
    #MACRO "KITA/FNC_3D"
    #name_3d = "ekth";
    return c_3d
  
#/*  Eith    */
#/* i(x)軸回りに a_3d を th_3d [deg] 回転(反時計回) => c_3d  */
def eith(a_3d,th_3d):
    e_3d[0][0] = 1.0         ; e_3d[0][1] = 0.0        ; e_3d[0][2] = 0.0
    e_3d[1][0] = 0.0         ; e_3d[1][1] = math.cos(th_3d) ; e_3d[1][2] = -math.sin(th_3d)  
    e_3d[2][0] = 0.0         ; e_3d[2][1] = math.sin(th_3d) ; e_3d[2][2] = math.cos(th_3d)
    mtrx3(a_3d,e_3d)
    #name_3d = "mtrx3";
    #MACRO "KITA/FNC_3D"
    #name_3d = "eith";
    return c_3d
  
#/*  Ejth    */
#/* j(y)軸回りに a_3d を th_3d [deg] 回転(反時計回) => c_3d  */
def ejth(a_3d,th_3d):
    e_3d[0][0] = math.cos(th_3d); e_3d[0][1] = 0.0 ; e_3d[0][2] = math.sin(th_3d);
    e_3d[1][0] = 0.0         ; e_3d[1][1] = 1.0    ; e_3d[1][2] = 0.0;  
    e_3d[2][0] = -math.sin(th_3d); e_3d[2][1] = 0.0; e_3d[2][2] = math.cos(th_3d);
    mtrx3(a_3d,e_3d)
    #name_3d = "mtrx3";
    #MACRO "KITA/FNC_3D"
    #name_3d = "ejth";
    return c_3d

# Ewth(R)
# Vector v0 を　w 軸(単位ベクトル)を中心に th(rad) 回転して v を返す。
# w軸方向から見て反時計回りを正とする。
def ewth(v0,w,th):
   # import numpy as np
    r = w[0]; m = w[1]; n = w[2]      #r:λ m:μ n:ν
    j1 = np.matrix([[r*r,r*m, n*r],[r*m,m*m, m*n],[n*r,m*n,n*n]])
    j2 = np.matrix([[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0]]) - j1
    j3 = np.matrix([[0,-n,m],[n,0,-r],[-m,r,0]])
    vm = np.dot((j1 + j2*np.cos(th) + j3*np.sin(th)), v0)
    #a11 = np.cos(th)+r*r*(1-np.cos(th))
    #a12 = r*m*(1-np.cos(th))-n*np.sin(th)
    #a13 = n*r*(1-np.cos(th))+m*np.sin(th)
    #a21 = r*m*(1-np.cos(th))+n*np.sin(th)
    #a22 = np.cos(th)+m*m*(1-np.cos(th))
    #a23 = m*n*(1-np.cos(th))-r*np.sin(th)
    #a31 = n*r*(1-np.cos(th))-m*np.sin(th)
    #a32 = m*n*(1-np.cos(th))+r*np.sin(th)
    #a33 = np.cos(th)+n*n*(1-np.cos(th))
    #x = v0[0]; y = v0[1]; z = v0[2];
    #v = np.array([(a11*x+a12*y+a13*z),(a21*x+a22*y+a23*z),(a31*x+a32*y+a33*z)])
    #vm0 = np.matrix([[a11,a12,a13],[a21,a22,a23],[a31,a32,a33]])
    #vm = np.dot(vm0,v0)
    v = np.array(vm).ravel()
    return v




#/*	Matrix積	*/
#/*     c_3d,      e_3d,     a_3d	*/
#/*	c0   | e00 e01 e02 |  a0  	*/
#/*	c1 = | e10 e11 e12 |  a1  	*/
#/*	c2   | e20 e21 e22 |  a2  	*/

def mtrx3(a_3d,e_3d):
    c_3d[0] = e_3d[0][0]*a_3d[0] + e_3d[0][1]*a_3d[1] + e_3d[0][2]*a_3d[2];
    c_3d[1] = e_3d[1][0]*a_3d[0] + e_3d[1][1]*a_3d[1] + e_3d[1][2]*a_3d[2];
    c_3d[2] = e_3d[2][0]*a_3d[0] + e_3d[2][1]*a_3d[1] + e_3d[2][2]*a_3d[2];
    return c_3d


#-----------------------------------------------
#   平面ベクトル関係
#-----------------------------------------------
def r2p_rad_l(*vr):     #未完成
    """ 直交座標Vector -> 極座標系Vector 変換 (np.array rad表記)複数個を扱える 戻り値はリスト"""
    vp = [] 
    if np.ndim(vr) == 1:
        r = np.sqrt(vr[0]**2+vr[1]**2)
        th = np.arctan2(vr[1],vr[0])
        vp = [r,th]
    else:
        for i in range(len(vr)):
            r = np.sqrt(vr[i][0]**2+vr[i][1]**2)
            th = np.arctan2(vr[i][1],vr[i][0])
            vp.append([r,th])
  
    return vp
 
def r2p_deg_l(*vr):     #未完成
    """ 直交座標Vector -> 極座標系Vector 変換 (np.array) deg表記、複数個を扱える 戻り値はリスト"""
    vp = []
    #  print('len(vr)=',len(vr)) 
    if np.ndim(vr) == 1:        # 1次元配列、ベクトルが一個
        r = np.sqrt(vr[0]**2+vr[1]**2)
        th = np.arctan2(vr[1],vr[0])*180/np.pi
        vp = [r,th]
    else:                       # 多次元配列、ベクトルが複数個
        for i in range(len(vr)):
            r = np.sqrt(vr[i][0]**2+vr[i][1]**2)
            th_deg = np.arctan2(vr[i][1],vr[i][0]) *180/np.pi
            vp.append([r,th_deg])
  
    return vp

def p2r_l(*vp):         #未完成
    """ 極座標系Vectorのリスト -> 直交座標Vector 変換 (r,th[rad]) 複数個を扱える 戻り値はndarray"""
    vr = []
    #print('len(vp)=',len(vp))
    if np.ndim(vp) == 1:        # 1次元配列、ベクトルが一個
        r = vp[0]; th = vp[1]
        #arr = [r*np.cos(th),r*np.sin(th)]
        arr = np.array([r*np.cos(th),r*np.sin(th)])
    else:                       # 多次元配列、ベクトルが複数個
        for i in range(len(vp)):
            r = vp[i][0]; th = vp[i][1]
            #vr.append([r*np.cos(th),r*np.sin(th)])
            vr.append(np.array([r*np.cos(th),r*np.sin(th)]))
  
        arr = vr      
    #print('len(vr)=',len(vr))
    return arr
  
def p2r(vp):    # 極座標系Vector -> 直交座標Vector 変換 (r,th[rad])  戻り値はnp.array
    r = vp[0]; th = vp[1]
    #arr = [r*np.cos(th),r*np.sin(th)]
    arr = np.array([r*np.cos(th),r*np.sin(th)])

    return arr
  
def rotation(vr, th):      # Vector(直交座標) の回転 (th[rad], np.array)
    # 回転行列
    R = np.array([[np.cos(th), -np.sin(th)],
                  [np.sin(th),  np.cos(th)]])

    return np.dot(R, vr)
    #vp0 = r2p_rad(vr)
    #vp1 = [vp0[0],(vp0[1]+th)]
    #vr1 = p2r(vp1)
    #return vr1




def case2(va, vb, vc):   # 直線_vbと直線_vcの交点
    """ 直線と直線の交点
                ra tha   rb thb   rc  thc
      case 2    O   O     ?  O    ?   O  :直線と直線の交点(np.array)
      引数(vp:3ベクトルのリストvp[va,vb,vc]
      戻り値(heimen_v_n, (va,vb[],vc[])) vb, vcはリストであることに注意!
    """
    vrb = []; vrc = []
    #print(rp,vp)
    ra = np.sqrt(va[0]**2+va[1]**2); tha = np.arctan2(va[1],va[0])
    rb = np.sqrt(vb[0]**2+vb[1]**2); thb = np.arctan2(vb[1],vb[0])
    rc = np.sqrt(vc[0]**2+vc[1]**2); thc = np.arctan2(vc[1],vc[0])

    #print('tha = %2.5f thb = %2.5f  thc = %2.5f' % (tha, thb, thc))

    if np.sin(thb-thc) == 0:    # 平行線
        heimen_v_n = 0
        vrb.append(vb); vrb.append(vb)
        vrc.append(vc); vrc.append(vc)
        vr = (va, vrb, vrc)
    else:       # 
        rb = abs(ra * np.sin(tha-thc) / np.sin(thb-thc))
        rc = abs(ra * np.sin(tha-thb) / np.sin(thc-thb))
        tmpb = p2r([rb,thb])  # 極座標系Vector -> 直交座標Vector 変換
        tmpc = p2r([rc,thc])  #print(vp0)
        #if va[0] + tmpb[0] == tmpc[0]:
        gosa = 0.00001
        if abs(va[0] + tmpb[0] - tmpc[0]) < gosa:
            vrb.append(tmpb); vrb.append(tmpb)
            vrc.append(tmpc); vrc.append(tmpc)
        elif abs(va[0] + tmpb[0] + tmpc[0]) < gosa:
            print('vcを反転します')
            vrb.append(tmpb); vrb.append(tmpb)
            vrc.append(-tmpc); vrc.append(-tmpc)
        elif abs(va[0] - tmpb[0] - tmpc[0]) < gosa:
            print('vbを反転します')
            vrb.append(-tmpb); vrb.append(-tmpb)
            vrc.append(tmpc); vrc.append(tmpc)
        elif abs(va[0] - tmpb[0] + tmpc[0]) < gosa:
            print('vb, vcを反転します')
            vrb.append(-tmpb); vrb.append(-tmpb)
            vrc.append(-tmpc); vrc.append(-tmpc)

        #print(len(vrb), len(vrc))
        vr = (va, vrb, vrc)
        heimen_v_n = 1

    return (heimen_v_n, vr)    

def case3(va, vb, vc):   # 直線_vbと円_vcの交点
    """ 直線と円の交点
                ra tha   rb thb   rc  thc
      case 3    O   O     O  ?2   ?2   O  :直線(c)と円(b)の交点(np.array)
      引数  va,vb,vc
      戻値  heimen_v_n:解の数, vra, vrb[], vrc[]
    """
    ra = np.sqrt(va[0]**2+va[1]**2); tha = np.arctan2(va[1],va[0])
    rb = np.sqrt(vb[0]**2+vb[1]**2); thb = np.arctan2(vb[1],vb[0])
    rc = np.sqrt(vc[0]**2+vc[1]**2); thc = np.arctan2(vc[1],vc[0])
    
    snthbc = -ra * np.sin((tha-thc)) / rb
    hantei = 1 - snthbc**2
    #print(tha, thc)
    #print(hantei)
    vrb = []; vrc = []

    if hantei < 0:      # 交点無し
        heimen_vct_n = 0
        vr = (va, vb, vc)

    elif hantei == 0:    # 接線(重根)
        thb = thc + np.pi/2    # [rad]  
        #thb0 = thc + 90;
        vra = va
        vrb = p2r(np.array([rb, thb]))
        vrc = vra + vrb
        heimen_vct_n = 1
        vr = (vra, vrb, vrc)
         
        
    else:
        csthbc = np.sqrt(hantei);
        thb = []; rc = []
        thb.append(thc+(np.arctan2(snthbc,csthbc)))
        thb.append(thc+(np.arctan2(snthbc,-csthbc)))
        rc.append(rb*np.cos(thb[0]-thc) + ra*np.cos(tha-thc))
        rc.append(rb*np.cos(thb[1]-thc) + ra*np.cos(tha-thc))
        
        #print('thb1 = %2.5f  thb2 = %2.5f' % (thb[0],thb[1]))
        #print('rc1 = %2.5f  rc2 = %2.5f' % (rc[0],rc[1]))
        vra = va
        vrb = []
        vrb.append(p2r([rb,thb[0]]))
        vrb.append(p2r([rb,thb[1]]))
        vrc = []
        vrc.append(p2r([rc[0],thc]))
        vrc.append(p2r([rc[1],thc]))
   
        vr = (vra, vrb, vrc)
        heimen_vct_n = 2
    
    return (heimen_vct_n, vr)

def case4(va, vb, vc):   #円_vbと円_vcの交点
    ra = np.sqrt(va[0]**2+va[1]**2); tha = np.arctan2(va[1],va[0])
    rb = np.sqrt(vb[0]**2+vb[1]**2); thb = np.arctan2(vb[1],vb[0])
    rc = np.sqrt(vc[0]**2+vc[1]**2); thc = np.arctan2(vc[1],vc[0])
    
    #print(tha, thb, thc)
    csthba = (rc**2-rb**2-ra**2)/(2*rb*ra)
    hantei = 1 - csthba*csthba
    vra = va
    if hantei < 0:      # 交点無し 
        heimen_vct_n = 0
        vrb = vb, vrc = vc
  
    elif hantei == 0: # 接線(重根)
        vrb = -vra * LA.norm(vb)
        vrc = vra * LA.norm(vc)
        
        #vrb = p2r(np.array([rb, -tha]))
        #vrc = vra + vrb
        heimen_vct_n = 1
        #vr = (va, vb, vc)

    
    elif hantei > 0:
        thb = []; thc = [] 
        snthba = np.sqrt(hantei)
        thb.append(tha + np.arctan2(snthba,csthba))
        thb.append(tha + np.arctan2(-snthba,csthba))
        thc.append(np.arctan2((rb*np.sin(thb[0]) + ra*np.sin(tha)),(rb*np.cos(thb[0]) + ra*np.cos(tha))))
        thc.append(np.arctan2((rb*np.sin(thb[1]) + ra*np.sin(tha)),(rb*np.cos(thb[1]) + ra*np.cos(tha))))

        #print('thb=%3.3f  thc=%3.3f' % (thb[0]/np.pi*180, thc[0]/np.pi*180))

        vrb = []
        vrb.append(p2r([rb,thb[0]]))
        vrb.append(p2r([rb,thb[1]]))
        vrc = []
        vrc.append(p2r([rc,thc[0]]))
        vrc.append(p2r([rc,thc[1]]))
   
        #vr = (vra, vrb, vrc)
       
        heimen_vct_n = 2
    
    return (heimen_vct_n, vra, vrb, vrc)    # 解の個数 ベクトル vra, vrb[], vrc[] 


def check_parallel2D(v0, v1):   # v0とv1が平行かどうかをチェック
    gosa= 0.000000001   # 10^-8
    tmp= vvth_pm(v0, v1)   # 内角±
    #print(tmp)  ####
    if abs(tmp)< gosa or (np.pi-abs(tmp))< gosa:
        return True
    else:
        return False
    
# e^jθを返す関数　戻り値は複素数(Complex)
def ej(th):  # th [rad] 
    c= cmath.rect(1, th)
    return c

# 複素数(Complex)をnp.array Vectorに変換
def c2v(c):  #c [complex]
    v= np.array([c.real, c.imag])
    return v




#------------------------------
# メインルーチン
#------------------------------
if __name__ == '__main__':
    if len(sys.argv) < 2:   #引き数無し !!!引数無
        menu = 3
    else:    # 引数１個 
        menu = int(sys.argv[1])

    if menu== 15:   # check_parallel2D(v0,v1)
        v0= np.array([100.0, 0.0])
        v1= np.array([-200.0, 0.0])
        v2= np.array([100.0, 0.00001])
        if check_parallel2D(v0, v1):
            print('v0とv1は平行です')
        else:
            print('v0とv1は平行ではありません\007')
        if check_parallel2D(v0, v2):
            print('v0とv2は平行です')
        else:
            print('v0とv2は平行ではありません\007')
    
    if menu == 14: #l2pvec(p0, p1, p2) return (px, v0x_, vx2_)
        p0= np.array([0.0,0.0])
        p1= np.array([720.0,100.0])
        p2= np.array([720.0,-100.0])
        (px, v0x_, vx2_)= l2pvec(p0,p1,p2)
        print(px, v0x_, vx2_)

    if menu == 13:  # 平面と点の距離 f2plen(v0_, p0, p1)
        v0 = np.array([0.0, 19.31852, -5.17638])
        v0_ = v0/LA.norm(v0)
        p0 = np.array([10.0,10.0,10.0])
        p1 = np.array([12.05514, 31.36715, 12.46921])
        ll = round(f2plen(v0_, p0, p1), 15) 
        print(ll)   # 20.000004145539595
        ll = round(f2plen(v0, p0, p1), 5)   # 少数点以下5桁で丸める 
        print(ll)   # 20.000004145539595    単位ベクトルじゃなくていい 

    if menu == 12:
        v = np.array([100.0, 100.0, 100.0])
        #v = [100.0, 100.0, 100.0]
        p = cartesian2polar(v)
        print(p)

    if menu == 11:  # def v2p3(v) 直交座標->極座標に変換
        v = np.array([100.0, 100.0, 100.0])
        (p_rad, p_deg) = v2p3(v)
        print(p_rad, p_deg)

    if menu ==10:   #vvth_pm(v0, v1)
        nn= 18
        for i in range(nn+1):
            th0= 2*i*np.pi/nn
            v0= np.array([100.0, 0.0])
            v1 = rotate2(v0,(th0-np.pi)) 
            #v1= rotate2(v0, th0) 
            th = vvth_pm(v0, v1)
            thd = np.rad2deg(th)
            print(' %2d %6.1f  %6.1f' % (i, np.rad2deg(th0), thd))
            #print(i, th, thd)
            
        

    if menu == 9:
        v0 = np.array([1.2,0.8])
        v1 = np.array([1.0,0.0])
        th = vvth(v0, v1)
        thd = np.rad2deg(th)
        print(th, thd)
        th = vvth(v1, v0)
        thd = np.rad2deg(th)
        print(th, thd)
        th = vvth(v0, -v1)
        thd = np.rad2deg(th)
        print(th, thd)
       

    if menu == 8:
        vr = np.array([1,0])
        v45 = rotation(vr, np.pi/4)
        print(v45)


    if menu == 7:   # p2r() 極座標系Vector -> 直交座標Vector 変換 (r,th[rad]) 複数個を扱える 戻り値はnp.array
        va = np.array([10.0, np.pi/4])
        vb = np.array([10.0, np.pi/3])
        vp = [va, vb]
        print(p2r(va))      # [array([7.07106781, 7.07106781])]
        print(p2r(vb))     # [array([7.07106781, 7.07106781]), array([5.0, 8.66025404])]   
        #vp1 = p2r(vp)
        #print(vp1[0], vp1[1])   # [7.07106781 7.07106781] [5.0, 8.66025404]
        #print(p2r(*vp)[0][0])   # 7.0710678118654755
        print(va + vb)      # [20.0 ,1.83259571]
        
    elif menu == 6:   #case4(va, vb, vc):   # 円と円の交点
        import icd_module as imd 
        
        va = np.array([10.0, 0.0])   #直交座標
        vb = np.array([8.0, 0.0])
        vc = np.array([0.0, 5.0])
        vr = [va,vb,vc]
        #print(vr)   # [array([10,  0]), array([5, 5]), array([0, 5])]
        (vn, vra, vrb, vrc) = case4(va, vb, vc)     # case4     
        print(vra)     # va  [10.  0.]
        print(vrb)  # vb1 [array([-6.95      ,  3.96200707])]
        print(vrc)  # vb2 [array([-6.95      , -3.96200707])]
        v0 = np.array([300.0, 0.0])   #直交座標
        imd.p2p_line(v0, v0+vra, '赤') 
        imd.p2p_line(v0+vra,v0+vra+vrb[0], '空') 
        imd.p2p_line(v0+vra,v0+vra+vrb[1], '空') 
        imd.p2p_line(v0, v0+vrc[0], '緑') 
        imd.p2p_line(v0, v0+vrc[1], '緑') 

    elif menu == 5:   #case2(va, vb, vc):   #直線と直線の交点
        va = np.array([10.0, 0.0])   #直交座標
        vb = np.array([5.0, 5.0])
        vc = np.array([0.0, 8.0])
        vr = [va,vb,vc]
        #print(vr)   # [array([10,  0]), array([5, 5]), array([0, 5])]
        (vn,(vra, vrb, vrc)) = case2(va, vb, vc)     # case2     
        print(vra, vrb, vrc) # [10.  0.] [10. 10.] [6.123234e-16 1.000000e+01]
        #print(v_abc[0], v_abc[1], v_abc[2]) # [10.  0.] [10. 10.] [6.123234e-16 1.000000e+01]

    elif menu == 4:   #case3(va, vb, vc):   #直線と円
        va = np.array([10.0, 0.0])   #直交座標
        vb = np.array([8.0, 0.0])
        vc = np.array([5.0, 5.0])
        vr = [va,vb,vc]
        (vn, (vra, vrb, vrc)) = case3(va,vb,vc)    # case3
        print(vra)     # va [10.  0.]
        print(vrb[0])  # vb1 [-2.35424869  7.64575131]
        print(vrb[1])  # vb2 [-7.64575131  2.35424869]
        print(vrc[0])  # vc1 [2.35424869 2.35424869]
        print(vrc[1])  # vc2 [2.35424869 2.35424869]
    
        


    elif menu == 3:       # ewth
        v0 = np.array([0.0,1.0,0.0])
        w = np.array([1.0,1.0,1.0])
        th = np.pi/4
        v = ewth(v0,w,th)
        print(v0,v)

    elif menu == 2: # rotate2
        v0 = np.array([0.0,1.0])
        th = np.deg2rad(90.0)
        v1 = rotate2(v0, th)
        print(v0,v1)

    elif menu == 1: # vvth
        v0 = np.array([1.0,0.0,0.0])    # 3D
        v1 = np.array([0.0,1.0,0.0])
        print(vvth(v0, v1))
        
        v0 = np.array([1.0,0.0])        # 2D
        v1 = np.array([0.0,1.0])
        print(vvth(v0, v1))
